//
//  PhobosDataManager.m
//  GMCache
//
//  Created by Locus on 2020/1/21.
//

#import "PhobosDataManager.h"
#import "PhobosSendManager.h"
#import "PhobosConfig.h"
#import "PhobosUtil.h"
#import <MJExtension/MJExtension.h>

@implementation PhobosDataManager

static NSManagedObjectContext *Phobos_defaultContext;

+ (void)initialize {
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        [GMCoreData setup];
        NSPredicate *predicate = [NSPredicate predicateWithFormat:@"status = %d", PhobosDataSendStatusSending];
        NSArray<PhobosDataEntity *> *entities = [self fetchDataEntitiesWithPredicate:predicate];
        [self updateDataEntities:entities sendStatus:PhobosDataSendStatusError completion:nil];
    });
}

+ (NSArray<PhobosDataEntity *> *)fetchToBeSendDataEntities {
    NSPredicate *predicate = [NSPredicate predicateWithFormat:@"status = %d or status = %d", PhobosDataSendStatusToBeSend, PhobosDataSendStatusError];
    return [self fetchDataEntitiesWithPredicate:predicate];
}

+ (NSArray<PhobosDataEntity *> *)fetchDataEntitiesWithPredicate:(NSPredicate *)searchFilter {
    return [PhobosDataEntity MR_findAllWithPredicate:searchFilter inContext:[NSManagedObjectContext MR_defaultContext]];
}

+ (NSUInteger)fetchCountOfToBeSendEntities {
    NSPredicate *predicate = [NSPredicate predicateWithFormat:@"status = %d or status = %d", PhobosDataSendStatusToBeSend, PhobosDataSendStatusError];
    return [self fetchCountOfEntitiesWithPredicate:predicate];
}

+ (NSUInteger)fetchCountOfEntitiesWithPredicate:(NSPredicate *)searchFilter {
    return [PhobosDataEntity MR_numberOfEntitiesWithPredicate:searchFilter inContext:[NSManagedObjectContext MR_defaultContext]];
}

+ (void)insertData:(NSDictionary *)data sendAPI:(NSString *)sendAPI completion:(MRSaveCompletionHandler)completion {
    if (!sendAPI || [sendAPI isEqualToString:@""] || !data) {
        return;
    }
    PhobosDataEntity *entity = [PhobosDataEntity MR_createEntity];
    entity.data = [data mj_JSONData];
    entity.api = sendAPI;
    entity.status = PhobosDataSendStatusToBeSend;
    entity.id = [PhobosUtil currentMMTime];
    [[NSManagedObjectContext MR_defaultContext] MR_saveWithOptions:MRSaveSynchronously completion:completion];
}

+ (void)updateDataEntities:(NSArray<PhobosDataEntity *> *)entities sendStatus:(PhobosDataSendStatus)sendStatus completion:(MRSaveCompletionHandler)completion {
    if (entities.count > 0) {
        [entities enumerateObjectsUsingBlock:^(PhobosDataEntity *obj, NSUInteger idx, BOOL * _Nonnull stop) {
            obj.status = sendStatus;
        }];
        [[NSManagedObjectContext MR_defaultContext] MR_saveWithOptions:MRSaveSynchronously completion:completion];
    }
}

+ (void)deleteDataEntities:(NSArray<PhobosDataEntity *> *)entities {
    if (entities.count > 0) {
        [entities enumerateObjectsUsingBlock:^(PhobosDataEntity *obj, NSUInteger idx, BOOL * _Nonnull stop) {
            [obj MR_deleteEntity];
        }];
        [[NSManagedObjectContext MR_defaultContext] MR_saveOnlySelfAndWait];
    }
}

+ (void)deleteAllEntities {
    [PhobosDataEntity MR_truncateAll];
    [[NSManagedObjectContext MR_defaultContext] MR_saveOnlySelfAndWait];
}

@end
